#removes all unimputable and low confidence imputable SNPs
#also standarizes format for all other plumage scripts


import argparse
import pandas, os, re, sys

def numeric_value(s):
    m = re.findall(r'.*?_(\d+)', s)
    return int(m[0]) if m else 0

def filter_imputed(i, s, o):

    strin= s
    if 1 < strin:
        raise Exception("AttributeError: filter stringency (-s) must be between 0 and one")

    hapmap= pandas.read_table(i, sep="\t", index_col=None, na_values= ["NA", "N"])
    hapmap=hapmap.drop(['Unnamed: 0'], axis= 1)
    hapmap= hapmap.rename(columns= {'snp_id':'rs#', 'chr':'chrom'})
    
    hapmap=hapmap.set_index('rs#',drop=False)

    old_count=len(hapmap)
    out= hapmap[hapmap['accuracy estimate'] >= strin]

    new_count=len(out)
        
    out= out.reindex(columns=sorted(out.columns, key= numeric_value))
    out= out.sort_index(by=['chrom','pos'])

    
    out.to_csv(o, sep='\t', na_rep="NA",index=False)
    dropped= old_count-new_count
    percent_dropped= (dropped/old_count)*100
    print(str(dropped)+"/"+str(old_count)+" snps with imputation accuracy <" +str(s)+
          " = " + str(percent_dropped)+ "% dropped", file=sys.stderr)


def main():
    
    
    parser= argparse.ArgumentParser(description="PLUMAGE v. 2.0: pipeline for GBS error correction")

    parser.add_argument('-i', metavar= 'infile', type= argparse.FileType('r'), nargs='?', default=sys.stdin,
                        help= 'file containing imputation ouput, default is stdin')

    parser.add_argument('-s', metavar= 'filter_stringency', type= float,
                        help= 'imputation confidence score (0 < s <=1) below which SNP will be discarded.\n Default = .95',
                        default= .95)

    parser.add_argument('-o', metavar= 'outfile', nargs='?', type= argparse.FileType('w'), default=sys.stdout,
                        help= 'path/filename for file you want filtered results written to, default= stdout')

    args = parser.parse_args()
    filter_imputed(args.i, args.s, args.o)
    
    print("Plumage 1 finished", file=sys.stderr)

if __name__ == "__main__":
  main()


    
    



