#The purpose of this script is to change untrustworthy calls to "NA"
# untrustworthy calls can be caused by sequencing errors or mismapping of tags
#script first id's recombination breakpoints, then determines if the call is likely to be an error
#if the next three calls on different tags match the breakpoint call, it is left as is
#if the next three calls do not match, the breakpoint call is changed to "NA"
#Format issues: make sure cols rs#, alleles, chrom, and pos are the first five cols in the spreadsheet
#and all the rest of the cols have individuals 

import argparse
import pandas, os, re, sys
import numpy as np

def numeric_value(s):
    m = re.findall(r'.*?_(\d+)', s)
    return int(m[0]) if m else 0

def is_nan(s):
    is_null=False
    if pandas.isnull(s): is_null=True
    elif (s is np.nan): is_null=True
    elif s.startswith("NaN_"): is_null=True
    return np.nan if is_null else s

def test_break(series, start):
    orig_val = series[start]
    orig_tag = start // 64
    
    count = 0
    for bp, val in series.ix[start:].iteritems():    
        if val != orig_val:
            return False
        if (bp // 64) > orig_tag:
            count += 1
        if count >= 4:
            return True
    return False

def error_correction(i, p1, p2, o, e):

    bad_report=[]
    full_report=[]
    corrections= 0
    s_correct=[]
    all_orig=[]

    hmp= pandas.read_table(i, na_values=['NA','N'])
    c_grouped=hmp.groupby('chrom')

    for chrom, hmp in c_grouped:

        hmp= hmp.drop(['accuracy estimate','pre-impute-missing','post-impute-missing','imputation.note'],axis=1)
        hmp= hmp.drop_duplicates(['rs#'])

        hmp1= hmp.set_index('pos', drop=False)
        hmp2= hmp.set_index('pos', drop=False)

        #change various NA notation to simply "NA" and drop markers that have parent hets
        first_cols= hmp2.ix[:, 0:4] 
        second= (hmp2.ix[:, 4:]).applymap(is_nan)
        second= second.applymap(lambda v: np.nan if v in ('M', 'R', 'W', 'S', 'Y', 'K') else v)
    
        hmp2= first_cols.join(second, how= 'outer')
        hmp2= hmp2.dropna(axis= 0, subset= [p1, p2])
        hmp2= hmp2.drop_duplicates(['pos'])

        #recode according to parent alelle 
        for row_idx, row in hmp2.iterrows():
            hmp2.ix[row_idx, row == row[p1]]= 'A'
            hmp2.ix[row_idx, row == row[p2]]='H'

        #Workhouse loop - changes untrustworthy calls to NA
    
        for col, series in hmp2.ix[0:,5:].iteritems():                                   
            last = None                                                                  
            for row_idx, val in series.iteritems():                                      
                is_good_break= (last != None and val != last and test_break(series, row_idx)==True)   
                if is_good_break or not last:                                            
                    last = val
                if val != last:                                                 
                    series[row_idx] = "NA"
                    info= [hmp2.ix[row_idx,['rs#']],hmp2.ix[row_idx,['pos']], col]
                    bad_report.append(info)
                    corrections += 1
                    s_correct.append([row_idx,col])

        for entry in s_correct:
            p= entry[0]
            c= entry[1]
            hmp1.ix[p,c]= "NA"
    
        s_correct= []
        all_orig.append(hmp1)

    #concat hapmaps and export files
    if e != None:
        Df_report= pandas.DataFrame(bad_report)
        Df_report.to_csv(e, sep= '\t', index=False)

    out_orig= pandas.concat(all_orig)
    
    out_orig= out_orig.sort_index(by=['chrom','pos'])
    out_orig= out_orig.reindex(columns= sorted(out_orig.columns, key= numeric_value))

    old_cols= out_orig.columns
    lst=[]
    old_cols= out_orig.columns
    for entry in old_cols[6:]:
        lst.append(entry)
    new_cols= ['rs#', 'alleles', 'chrom', 'pos', 'IR64','Azucena']+lst
    out_orig= out_orig.reindex(columns= new_cols)
    out_orig.to_csv(o, sep='\t', index=False, na_rep="NA")

                  
    print("changed "+str(corrections) + " calls to NA",file=sys.stderr)

def main():
    
    parser= argparse.ArgumentParser(description= "PLUMAGE v. 2.0: pipeline for GBS error correction")

    parser.add_argument('-i', metavar= 'infile', type=argparse.FileType('r'), nargs='?', default=sys.stdin,
                        help= 'path/file containing output of Plumage 1, default = stdin')

    parser.add_argument('-p1', metavar= 'parent_1', help= "exact name of parent 1 as it is in your files",
                        required= True)

    parser.add_argument('-p2', metavar= 'parent_2', help= "exact name of parent 2 as it is in your files",
                        required= True)

    parser.add_argument('-o', nargs= '?', metavar= 'outfile', type=argparse.FileType('w'),
                        default= sys.stdout, help= "optional file path/name if you want to save results of this script\n"
                        "default = stdout")

    parser.add_argument('-e', nargs='?', metavar= 'error_report', default= None,
                        help= "specify a file path/name if you want to save a report of sequencing errors")

    args = parser.parse_args()
    error_correction(args.i, args.p1, args.p2, args.o, args.e) 

    
    print("Plumage 2 finished!", file=sys.stderr)

if __name__ == "__main__":
  main()


