import argparse, pandas, os, re, sys
import numpy as np

def remove_failed(i, drop_m, drop_i, o):
    infile= i
    hmp= pandas.read_table(infile)
    pop_size= len(hmp.ix[0])-4
    old_c=len(hmp)

    hmp['marker_call_rate'] = (pop_size - (pandas.isnull(hmp.ix[0:,4:]).sum(axis=1)))/pop_size
    individuals= ((pandas.isnull(hmp).sum())/len(hmp))

    hmp_trunc= hmp[hmp['marker_call_rate'] >= drop_m]
    new_c= len(hmp_trunc)
    c_dropped= old_c-new_c
    p_dropped= (c_dropped/len(hmp))*100
    print("dropped " + str(c_dropped) + " markers with call rates below "  + str(drop_m) +
          ", =" + str(p_dropped) +"% markers dropped", file=sys.stderr)

    individuals= individuals[individuals > drop_i]
    hmp_trunc= hmp_trunc.drop(individuals.index, axis= 1)

    hmp_trunc.to_csv(o, sep="\t", index=False, na_rep= "NA")

    if len(individuals) != 0:
        print("the following individuals were dropped: \n", file=sys.stderr)
        print(individuals, file= sys.stderr)

    

def main():
    parser= argparse.ArgumentParser(description= "PLUMAGE v. 2.0: pipeline for GBS error correction")
    parser.add_argument('-i', metavar= 'infile', help= 'path/name if you want to import a specific file, default= stdin',
                        nargs ='?', default= sys.stdin, type= argparse.FileType('r'))

    parser.add_argument('--drop_m', metavar='marker_threshold', type= float, default= .75,
                        help= "marker call rate below which marker is dropped, 0 < m <= 1, default is .75")

    parser.add_argument('--drop_i', type= float, metavar='individual_threshold', default= 1,
                        help= "proportion missing data at and above which individuals should be dropped, default keep all, 0 < i <= 1 \n"+
                        "if drop_i is set to .2, individuals with more than 20 percent missing data will be dropped.")

    parser.add_argument('-o', metavar= 'outfile', help= "path/name for filter results, default= stdout",
                        nargs= '?', type= argparse.FileType('w'), default= sys.stdout)

    args = parser.parse_args()
    remove_failed(args.i, args.drop_m, args.drop_i, args.o)
    print("Plumage 3 is finished", file= sys.stderr)


if __name__ == "__main__":
  main()
