import argparse, pandas, os, re, sys
import numpy as np

def is_nan(s):
    is_null=False
    if pandas.isnull(s): is_null=True
    elif (s is np.nan): is_null=True
    elif s.startswith("NaN_"): is_null=True
    return np.nan if is_null else s

def numeric_value(s):
    m = re.findall(r'.*?_(\d+)', s)
    return int(m[0]) if m else 0

def transform_RIL(i, o, has_rn, no_cr, p1, p2):
    lst=[]

    hmp= pandas.read_table(i,na_values=['N'])

    if has_rn == True and no_cr == False:
        hmp=hmp.drop(['alleles','read_num','marker_call_rate'], axis=1)

    elif has_rn == False and no_cr == False:
       hmp=hmp.drop(['alleles', 'marker_call_rate'],axis=1)

    elif has_rn == True and no_cr == True:
        hmp=hmp.drop(['alleles', 'read_num'], axis=1)

    else:
        hmp= hmp.drop(['alleles'],axis=1)

    first_cols= hmp.ix[0:, 0:3]
    second= (hmp.ix[0:, 3:])
    second= second.applymap(lambda v: np.nan if v in ('M', 'R', 'W', 'S', 'Y', 'K') else v)
    second=second.applymap(is_nan)

    hmp= first_cols.join(second, how='outer')
    hmp= hmp.dropna(subset= [p1, p2])

    for row_idx, row in hmp.iterrows():
        if hmp.ix[row_idx, p1] == hmp.ix[row_idx, p2]:
            lst.append(row_idx)
                       
        hmp.ix[row_idx, row == row[p1]]= 'A'
        hmp.ix[row_idx, row == row[p2]]='H'

    hmp= hmp.drop(lst)
    hmp= hmp.drop([p1, p2],axis=1)
    hmp= hmp.sort_index(by=['chrom','pos'], ascending= True)
    hmp= hmp.drop(['pos'], axis=1)
 
    hmp.to_csv(o, index=False, na_rep="NA")

def main():

    parser= argparse.ArgumentParser(description= "PLUMAGE v. 2.0: pipeline for GBS error correction")

    parser.add_argument('-i', metavar= 'infile', default= sys.stdin, nargs='?', type= argparse.FileType('r'),
                        help= "path/name for data file you want to format for r/qtl, default stdin")

    parser.add_argument('-o', metavar= 'outfile', required= True,
                        help= "path/name for resulting r/qtl formatted file")
    
    parser.add_argument('--has_rn', action= 'store_true',
                        help= "flag presence indicates that the infile has a column containing read number")

    parser.add_argument('--no_cr', action= 'store_true', 
                        help= "flag presence indicates infile has no column for call rates")

    parser.add_argument('-p1', required= True, metavar= "parent_1",
                        help= "enter name of parent 1 exactly as it appears in file")

    parser.add_argument('-p2', required= True, metavar= "parent_2",
                        help= "enter name of parent 2 exactly as it appears in file")

    args= parser.parse_args()
    transform_RIL(args.i, args.o, args.has_rn, args.no_cr, args.p1, args.p2)
    print("Plumage 5 is finished", file=sys.stderr)

if __name__ == "__main__":
  main()
                        
                        
                        
                        
                        

