import argparse, pandas, sys

def enrich_locs(i, o, k):

    hmp= pandas.read_table(i)
    locs= pandas.read_table(k)

    pieces={}

    for row_idx, row in locs.iterrows():
        pieces[row_idx]= hmp[(hmp['chrom'] == row['chrom'])&
                       (hmp['pos'] >= row['pos_start'])&
                       (hmp['pos'] <= row['pos_end'])]

    out= pandas.concat(pieces)

    out.to_csv(o, sep= '\t', index=False, na_rep="NA")
    

def main():
    parser= argparse.ArgumentParser(description="PLUMAGE v. 2.0: pipeline for GBS error correction")

    parser.add_argument('-i', metavar= 'infile', type= argparse.FileType('r'), default=sys.stdin,
                        help= "path/name for data file to choose subset from, default= stdin", nargs='?')

    parser.add_argument('-o', metavar= 'outfile', type= argparse.FileType('w'), default= sys.stdout, nargs='?',
                        help= "path/name for resulting subset, default= stdout")

    parser.add_argument('-k', metavar= 'location_key', required=True, type= argparse.FileType('r'),
                        help= "path/name for tab-delimited file containing three columns with names\n'chrom',"+
                        " 'pos_start', and 'pos_end', delineating regions for which you want all"+
                        "the snps in the file. Entered positions will be included in range.")

    args = parser.parse_args()
    enrich_locs(args.i, args.o, args.k)

    print("Plumage 7 is finished", file=sys.stderr)

if __name__ == "__main__":
  main()

